﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Cryptography;
using System.Text;
using System.Threading.Tasks;

namespace demo.Common
{
    public class SignHelper
    {
        /// <summary>
        /// 创建md5摘要,规则是:按参数名称a-z排序,遇到空值的参数不参加签名
        /// </summary>
        /// <param name="key">参数名</param>
        /// <param name="value">参数值</param>
        /// key和value通常用于填充最后一组参数
        /// <returns></returns>
        public static string CreateMd5Sign(Dictionary<string, object> dic, string key, string charset = "UTF-8")
        {
            var re = from pair in dic orderby pair.Key.ToLower() select pair;
            StringBuilder temp = new StringBuilder("");
            foreach (KeyValuePair<string, object> pair in re)
            {
                if (pair.Value == null || string.IsNullOrEmpty(pair.Value.ToString())) continue;
                if (pair.Key.CompareTo("sign") != 0)
                    temp.Append(pair.Key).Append("=").Append(pair.Value).Append("&");
            }
            string signStr = temp.ToString() + key;
            string sign = GetMD5(signStr, charset).ToUpper();

            return sign;
        }

        /// <summary>
        /// 签名验证
        /// </summary>
        /// <param name="sign">签名字符串</param>
        /// <param name="dic">待生成签名字典</param>
        /// <param name="appSecret">秘钥</param>
        /// <returns></returns>
        public static bool SignVerify(string sign, Dictionary<string, object> dic, string appSecret)
        {
            string newSign = CreateMd5Sign(dic, appSecret);
            return sign == newSign;
        }

        /// <summary>
        /// 获取大写的MD5签名结果
        /// </summary>
        /// <param name="encypStr"></param>
        /// <param name="charset"></param>
        /// <returns></returns>
        public static string GetMD5(string encypStr, string charset)
        {
            string retStr;
            MD5CryptoServiceProvider m5 = new MD5CryptoServiceProvider();

            //创建md5对象
            byte[] inputBye;
            byte[] outputBye;

            //使用GB2312编码方式把字符串转化为字节数组．
            try
            {
                inputBye = Encoding.GetEncoding(charset).GetBytes(encypStr);
            }
            catch (Exception ex)
            {
                inputBye = Encoding.GetEncoding("GB2312").GetBytes(encypStr);
            }
            outputBye = m5.ComputeHash(inputBye);

            retStr = System.BitConverter.ToString(outputBye);
            retStr = retStr.Replace("-", "").ToUpper();
            return retStr;
        }

    }
}
